'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = timerSafely;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var platform = typeof window === 'undefined' ? global : window;

var TIMER_TIMEOUT = 'timeout';
var TIMER_INTERVAL = 'interval';
var TIMER_IMMEDIATE = 'immediate';
var TIMER_RAF = 'raf';

function noop() {}

function setter(timer, clear, type) {
  return function (callback, delta) {
    var id = timer(callback, delta);

    if (!this[type]) {
      this[type] = [id];
    } else {
      this[type].push(id);
    }
    return id;
  };
}

function clearer(clear, type) {
  return function (id) {
    if (!this[type]) {
      return;
    }
    var index = this[type].indexOf(id);
    if (index !== -1) {
      this[type].splice(index, 1);
    }
    clear(id);
  };
}

var timerClearTimeout = clearer(platform.clearTimeout, TIMER_TIMEOUT);
var timerSetTimeout = setter(platform.setTimeout, timerClearTimeout, TIMER_TIMEOUT);

var timerClearInterval = clearer(platform.clearInterval, TIMER_INTERVAL);
var timerSetInterval = setter(platform.setInterval, timerClearInterval, TIMER_INTERVAL);

var timerClearImmediate = clearer(platform.clearImmediate, TIMER_IMMEDIATE);
var timerSetImmediate = setter(platform.clearImmediate, timerClearImmediate, TIMER_IMMEDIATE);

var timerCancelAnimationFrame = clearer(platform.cancelAnimationFrame, TIMER_RAF);
var timerRequestAnimationFrame = setter(platform.requestAnimationFrame, timerCancelAnimationFrame, TIMER_RAF);

function timerSafely(clazz) {

  // `timer` functions prepend `this.`
  clazz.prototype.setTimeout = timerSetTimeout;
  clazz.prototype.clearTimeout = timerClearTimeout;

  clazz.prototype.setInterval = timerSetInterval;
  clazz.prototype.clearInterval = timerClearInterval;

  clazz.prototype.setImmediate = timerSetImmediate;
  clazz.prototype.clearImmediate = timerClearImmediate;

  clazz.prototype.requestAnimationFrame = timerRequestAnimationFrame;
  clazz.prototype.cancelAnimationFrame = timerCancelAnimationFrame;

  var componentWillUnmount = clazz.prototype.componentWillUnmount || noop;

  clazz.prototype.componentWillUnmount = function () {

    // clear timer tasks
    this[TIMER_TIMEOUT] && this[TIMER_TIMEOUT].forEach(platform.clearTimeout);
    this[TIMER_TIMEOUT] = null;

    this[TIMER_INTERVAL] && this[TIMER_INTERVAL].forEach(platform.clearInterval);
    this[TIMER_INTERVAL] = null;

    this[TIMER_IMMEDIATE] && this[TIMER_IMMEDIATE].forEach(platform.clearImmediate);
    this[TIMER_IMMEDIATE] = null;

    this[TIMER_RAF] && this[TIMER_RAF].forEach(platform.cancelAnimationFrame);
    this[TIMER_RAF] = null;

    return componentWillUnmount.call(this);
  };

  return clazz;
}